!=========================================================================
! Copyright (C) GemTalk Systems 1986-2016.  All Rights Reserved.
!
! $Id$
!
! Definition of DateAndTime (from TimeZone2007.gs)
!
!=========================================================================

set class DateAndTime

removeallmethods 
removeallclassmethods 

! ------------------- Class methods for DateAndTime
category: 'For Documentation Installation only'
classmethod: DateAndTime
installDocumentation

self comment:
'This is the GemStone/S override to access System data.

This protocol describes the behavior that is common to date 
time objects. Date time objects represent individual points 
in Coordinated Universal Time (UTC) as represented in an
implementation defined local time.

The exact properties of local times are unspecified. Local 
times may differ in their offset from UTC. A given local time 
may have different offsets from UTC at different points in time.

All dates and times in the UTC local time are in the Gregorian 
calendar. Date times prior to the adoption of the Gregorian 
calendar are given in the retrospective astronomical Gregorian 
calendar. The year 1 B.C. is astronomical Gregorian year 0. 
The year 2 B.C. is astronomical Gregorian year-1. The year 1 A.D. 
is astronomical Gregorian year 1. The offset of the UTC local 
time is zero.'.
%

category: 'Overrides'
classmethod: DateAndTime
clockPrecision
	"Answer a <Duration> such that after that period of time passes, 
	#now is guaranteed to give a different result.
	Ideally implementations should answer the least such duration."

	| is64bit seconds |
	is64bit := 536870911 < SmallInteger maximumValue.
	seconds := is64bit
		ifTrue:  [1e-6]
		ifFalse: [1e-3].
	^Duration seconds: seconds.
%
category: 'Overrides'
classmethod: DateAndTime
secondsSince2001

	"^DateTime now asSecondsGmt - 3155760000."
	^System _timeGmtFloat - 978307200.
%
category: 'initialize-release'
method: DateAndTime
currentTimeZone

	^TimeZone current.
%
category: 'New Indexing Comparison'
method: DateAndTime
_classSortOrdinal

^ 70
%

! ------------------- Instance methods for DateAndTime
