!=========================================================================
! Copyright (C) GemTalk Systems 2011-2016.  All Rights Reserved.
!
! File GsSecureSocket.gs
!
!=========================================================================

doit
Globals at: #GsSecureSocket ifAbsent:[
  GsSocket _newKernelSubclass: 'GsSecureSocket'
    subclassOf: GsSocket
    instVarNames: #()
    classVars: #()
    classInstVars: #()
    poolDictionaries: #()
    inDictionary: Globals
	constraints: #()
    options:  #()
    reservedOop: 1945
].
true
%

set class GsSecureSocket
removeallmethods 
removeallclassmethods 

category: 'Documentation'
classmethod:
comment
^'GsSecureSocket provides the means for creating and binding TLS TCP sockets
 through the operating system of the machine that is running the session''s Gem process, and
 for communicating across those sockets.  Methods that block GemStone Smalltalk
 until the socket operation completes are interruptable by a hard break.  (You
 can get a hard break in Topaz by pressing the control-C key twice.  You can get
 a hard break in GemBuilder for C by calling the GciHardBreak function, and in
 GemBuilder for Smalltalk by calling the corresponding hard break method.)

 Like instances of GsSocket, instances of GsSecureSocket automatically have
 their C state closed when the instance is garbage collected or 
 when a persistent instance drops out of memory.

 SSLv2 and SSLv3 connections are not supported because such connections are known to 
 be insecure. Only and TLSv1.x connections are currently supported.

 To create a secure connection, a regular socket connection must first be established.
 Use the methods inherited from GsSocket to establish a socket connection.

                              Warning:
    Do not retain an instance of GsSecureSocket from one session to another.
    Instances of GsSecureSocket are intended to exist only within a given GemStone
    session.  GsSecureSockets that are used across sessions always generate an error.

 All instVars of GsSecureSocket are private, for use by the implementation
 of socket methods, and for use by the ProcessorScheduler only.

Constraints:
	fileDescriptor: SmallInteger
	lineNumber: SmallInteger
	readWaiters: Object
	writeWaiters: Object
	readyEvents: SmallInteger
	pollArrayOfs: SmallInteger
	selectWaiters: Object' .
%

category: 'Instance Creation'
classmethod:
basicNew

"disallowed"

self shouldNotImplement: #basicNew
%

classmethod: 
new: aSize
  self shouldNotImplement: #new:
%


classmethod: 
new
  self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod:
newServer

"Creates a new instance and initializes it to act as a AF_INET server socket.
 No connection is made."

^ super new initializeAsServer
%

category: 'Instance Creation'
classmethod:
newClient

"Creates a new instance and initializes it to act as a AF_INET client socket.
 No connection is made."

^ super new initializeAsClient
%
category: 'Instance Creation'
classmethod:
newServerIpv6

"Creates a new instance and initializes it to act as a AF_INET6 server socket.
 No connection is made."

^ super newIpv6 initializeAsServer
%

category: 'Instance Creation'
classmethod:
newClientIpv6

"Creates a new instance and initializes it to act as a AF_INET6 client socket.
 No connection is made."

^ super newIpv6 initializeAsClient
%

category: 'Initialization'
method:
initializeAsServer

"Returns the receiver"
^ self _zeroArgSslPrim: 13
%

category: 'Initialization'
method:
initializeAsClient

"Returns the receiver"
^ self _zeroArgSslPrim: 12
%

category: 'Private'
method:
_zeroArgSslPrim: opcode

"opcode  function
   1     instance method: secureConnect
   2     instance method: secureAccept
   3     instance method: hasSecureConnection
   4     instance method: secureClose
   5     instance method: fetchLastIoErrorString
   6        class method: fetchErrorStringArray
   7        class method: clearErrorQueue
   8        class method: disableCertificateVerificationOnClient
   9        class method: enableCertificateVerificationOnClient
  10        class method: disableCertificateVerificationOnServer
  11        class method: enableCertificateVerificationOnServer
  12     instance method: initializeAsClient
  13     instance method: initializeAsServer
  14        class method: fetchLastCertificateVerificationErrorForServer
  15        class method: fetchLastCertificateVerificationErrorForClient
  16     instance method: fetchCipherDescription
  17     instance method: disableCertificateVerification
  18     instance method: enableCertificateVerification
  19        class method: sslLibraryVersionString
  20     instance method: _peek
  21     instance method: certificateVerificationEnabled
  22        class method: certificateVerificationEnabledOnClient
  23        class method: certificateVerificationEnabledOnServer
  24     instance method: fetchCertificateVerificationOptions
  25        class method: fetchCertificateVerificationOptionsForServer
"
<primitive: 908>
^ self _primitiveFailed: #_zeroArgSslPrim: args: { opcode }
%

category: 'Private'
classmethod:
_zeroArgSslPrim: opcode

"opcode  function
   1     instance method: secureConnect
   2     instance method: secureAccept
   3     instance method: hasSecureConnection
   4     instance method: secureClose
   5     instance method: fetchLastIoErrorString
   6        class method: fetchErrorStringArray
   7        class method: clearErrorQueue
   8        class method: disableCertificateVerificationOnClient
   9        class method: enableCertificateVerificationOnClient
  10        class method: disableCertificateVerificationOnServer
  11        class method: enableCertificateVerificationOnServer
  12     instance method: initializeAsClient
  13     instance method: initializeAsServer
  14        class method: fetchLastCertificateVerificationErrorForServer
  15        class method: fetchLastCertificateVerificationErrorForClient
  16     instance method: fetchCipherDescription
  17     instance method: disableCertificateVerification
  18     instance method: enableCertificateVerification
  19        class method: sslLibraryVersionString
  20     instance method: _peek
  21     instance method: certificateVerificationEnabled
  22        class method: certificateVerificationEnabledOnClient
  23        class method: certificateVerificationEnabledOnServer
  24     instance method: fetchCertificateVerificationOptions
  25        class method: fetchCertificateVerificationOptionsForServer
"

<primitive: 908>
^ self _primitiveFailed: #_zeroArgSslPrim: args: { opcode }
%

category: 'Private'
classmethod:
_oneArgSslPrim: opcode with: arg

"opcode  function
   1        class method: useCACertificateFileForClients:
   2        class method: useCACertificateFileForServers:
   3        class method: setClientCipherListFromString:
   4        class method: setServerCipherListFromString:
   5     instance method: setCipherListFromString:
   6        class method: setCertificateVerificationOptionsForServer:
   7     instance method: setCertificateVerificationOptions:
   8        class method: useCACertificateDirectoryForClients:
   9        class method: useCACertificateDirectoryForServers:
  10     instance method: initializeAsServerFromGsSocket:
  11     instance method: initializeAsClientFromGsSocket:
"

<primitive: 910>
^ self _primitiveFailed: #_oneArgSslPrim:with: args: { opcode . arg }
%

category: 'Private'
method:
_oneArgSslPrim: opcode with: arg

"opcode  function
   1        class method: useCACertificateFileForClients:
   2        class method: useCACertificateFileForServers:
   3        class method: setClientCipherListFromString:
   4        class method: setServerCipherListFromString:
   5     instance method: setCipherListFromString:
   6        class method: setCertificateVerificationOptionsForServer:
   7     instance method: setCertificateVerificationOptions:
   8        class method: useCACertificateDirectoryForClients:
   9        class method: useCACertificateDirectoryForServers:
  10     instance method: initializeAsServerFromGsSocket:
  11     instance method: initializeAsClientFromGsSocket:
"

<primitive: 910>
^ self _primitiveFailed: #_oneArgSslPrim:with: args: { opcode . arg }
%

category: 'Private'
method:
_threeArgSslPrim: opcode with: arg1 with: arg2 with: arg3

"opcode  function
   1     instance method: _write:startingAt:ofSize:
   2     instance method: _readInto:startingAt:maxBytes:
   3        class method: useServerCertificate: withPrivateKey: privateKeyPassphrase:
   4        class method: useClientCertificate: withPrivateKey: privateKeyPassphrase:
   5     instance method: useCertificate: withPrivateKey: privateKeyPassphrase:
   6:       class method: useServerCertificateFile:withPrivateKeyFile:privateKeyPassphrase:
   7:       class method: useClientCertificateFile:withPrivateKeyFile:privateKeyPassphrase:
   8:    instance method: useCertificateFile:withPrivateKeyFile::privateKeyPassphrase:

"
<primitive: 909>
^ self _primitiveFailed: #_threeArgSslPrim:with:with:with: 
  args: { opcode . arg1 . arg2 . arg3 }
%


category: 'Private'
classmethod:
_threeArgSslPrim: opcode with: arg1 with: arg2 with: arg3

"opcode  function
   1     instance method: _write:startingAt:ofSize:
   2     instance method: _readInto:startingAt:maxBytes:
   3        class method: useServerCertificate: withPrivateKey: privateKeyPassphrase:
   4        class method: useClientCertificate: withPrivateKey: privateKeyPassphrase:
   5     instance method: useCertificate: withPrivateKey: privateKeyPassphrase:
   6:       class method: useServerCertificateFile:withPrivateKeyFile:privateKeyPassphrase:
   7:       class method: useClientCertificateFile:withPrivateKeyFile:privateKeyPassphrase:
   8:    instance method: useCertificateFile:withPrivateKeyFile::privateKeyPassphrase:

"
<primitive: 909>
^ self _primitiveFailed: #_threeArgSslPrim:with:with:with: 
  args: { opcode . arg1 . arg2 . arg3 }
%

! method comments edited for bug 42379 ; edit for fix 42667
category: 'Private'
method:
_write: aByteObject startingAt: anOffset ofSize: numBytes

"Does not conform exactly to GsSocket >> _write:startingAt:ofSize: 
 due to semantics of SSL renegotiation and socket buffering.

 Returns nil if a socket error occurred ,
 or a SmallInteger the number of bytes written which will
 normally be equal to numBytes."

[ true ] whileTrue:[
  | res |
  "Note: args must be the same each time if -1 or -2 is returned"
  res := self _threeArgSslPrim: 1 with: aByteObject with: anOffset with: numBytes .
  res == -2 ifTrue:[
    ^ false
  ] ifFalse:[ 
    res == -1 ifTrue:[ self _waitForReadReady ] "can happen due to SSL renegotiation"
              ifFalse:[ ^ res ].
  ].
].
%

category: 'Private'
method:
_primBlockingReadInto: aByteObject startingAt: anOffset maxBytes: numBytes

"Returns a SmallInteger indicating how many bytes were read or
 nil if an error occurs.
 Returns 0 if EOF occurs on the socket.

 Automatically handles conditions which require waiting for a read-ready
 or write-ready condition."

[ true ] whileTrue:[
  | res |
  "Note: args must be the same each time if -1 or -2 is returned"
  res := self _threeArgSslPrim: 2 with: aByteObject with: anOffset with: numBytes .
  res == -1 ifTrue:[self _waitForReadReady ] "Normal case"
            ifFalse:[ res == -2
                      ifTrue:[ self _waitForWriteReady ] "can happen due to SSL renegotiation"
                      ifFalse:[ ^ res ]. "done or error"
            ].
].
%

category: 'Private'
method:
_primNonBlockingReadInto: aByteObject startingAt: anOffset maxBytes: numBytes

"Returns a SmallInteger indicating how many bytes were read or
 nil if an error occurs.
 Returns 0 if EOF occurs on the socket.

 Automatically handles conditions which require waiting for a read-ready
 or write-ready condition."

[ true ] whileTrue:[
  | res |
  "Note: args must be the same each time if -1 or -2 is returned"
  res := self _threeArgSslPrim: 2 with: aByteObject with: anOffset with: numBytes .
  res == -1 ifTrue:[^ false "would block" ] 
            ifFalse:[ res == -2
                      ifTrue:[ self _waitForWriteReady ] "can happen due to SSL renegotiation"
                      ifFalse:[ ^ res ]. "done or error"
            ].
].
%

category: 'Private'
method:
_readInto: aByteObject startingAt: anOffset maxBytes: numBytes

"Read numBytes of data from the receiver into aByteObject starting at anOffset.
 Returns a SmallInteger indicating how many bytes were read,
 or returns 0 if EOF occurs on the socket,
 or signals an error."

| res |
res := self isNonBlocking 
   ifTrue:[ self _primNonBlockingReadInto: aByteObject startingAt: anOffset maxBytes: numBytes ]
  ifFalse:[ self _primBlockingReadInto: aByteObject startingAt: anOffset maxBytes: numBytes ].
res ifNil:[
  SocketError signal: self fetchLastIoErrorString
].
^ res
%

! fixed 46375
category: 'Server Operations'
method:
secureAccept

"Establishes a secure connection using the receiver as the server  The receiver
 must have already established a regular (non-secure) connection with its 
 client.  No certificate management is performed by this method.  A certificate
 and private key must be installed before calling this method.

 This method makes the following OpenSSL calls:

   SSL_new
   SSL_set_fd
   SSL_accept

 Returns true to indicate the operation was successful or false to indicate
 an error occurred."

[ true ] whileTrue:[
  | res |
  "Note: args must be the same each time if -1 or -2 is returned"
  res := self _zeroArgSslPrim: 2 .
  res == -1 ifTrue:[ Delay waitForMilliseconds: 20 ]
           ifFalse:[ ^ res ]. "done!"
].
%

category: 'Server Operations'
method:
accept

"Causes the receiver to perform a normal socket accept operation (see the 
 accept method in GsSocket for more information).  If the accept is
 successful, then it also initializes the newly created socket as an SSL
 server socket.

 Returns the new socket or nil if an error occurs."

| newServer |
newServer := super accept .
newServer ifNil:[ ^ nil ]. "Normal accept failed"
newServer initializeAsServer . "SSL init as a server socket"
^ newServer 
%

category: 'Server Operations'
method:
acceptTimeoutMs: timeoutMs

  | newServer |
  newServer := super acceptTimeoutMs: timeoutMs .
  newServer ifNil:[ ^ nil ]. "Normal accept failed"
  newServer initializeAsServer . "SSL init as a server socket"
  ^ newServer
%

! fix 46375
category: 'Client Operations'
method:
secureConnect

"Establishes a secure connection using the receiver as the client  The receiver
 must have already established a regular (non-secure) connection with its 
 server.  No certificate management is performed by this method.

 This method makes the following OpenSSL calls:

   SSL_new
   SSL_set_fd
   SSL_connect

 Returns true to indicate the operation was successful or false to indicate
 an error occurred.
"

[ true ] whileTrue:[
  | res |
  "Note: args must be the same each time if -1 or -2 is returned"
  res := self _zeroArgSslPrim: 1 .
  res == -1 ifTrue:[ Delay waitForMilliseconds: 20 ]
           ifFalse:[ ^ res ]. "done!"
].
%

category: 'Querying'
method:
hasSecureConnection

"Answer true if the receiver has established a secure connection
 with its peer.  Otherwise answer false."

^ self _zeroArgSslPrim: 3
%


category: 'Private'
method:
secureClose

"Closes the secure connection and releases memory used by the receiver's
 SSL connection.  Does not close the underlying socket connection.  Customers
 should use the #close method instead of this one unless the underlying 
 (insecure) socket connection is to be retained.

 Returns the receiver."

^ self _zeroArgSslPrim: 4
%

category: 'Error Handling'
method:
fetchLastIoErrorString

"Fetches a string describing the last I/O error encountered by a call to any
 of the following SSL functions:

   SSL_connect
   SSL_accept
   SSL_do_handshake
   SSL_read
   SSL_peek
   SSL_write

 Note that a result containing the substring
   SSL_write:uninitialized
 means that SSL_write was attempted before completing all of the
 handshakes of the sslConnect / sslAccept pair .

 The error is also cleared.  Returns nil if no I/O error has occurred
 on the receiver."

^ self _zeroArgSslPrim: 5
%

method:
lastErrorString
  "Provided for compatibility with GsSocket.
   See comments in fetchLastIoErrorString"
  
  ^ self fetchLastIoErrorString ifNil:[ super lastErrorString]
%

classmethod:
lastErrorString

  "Provided for compatibility with GsSocket.
   Returns a string describing the elements in the SSL error queue,
   and clears that queue.  Returns nil if no error is available"

| arr str |
arr := self fetchErrorStringArray .
arr size > 0 ifTrue:[
  str := String new .
  1 to: arr size do:[:j | str add: (arr at: j); add: '; ' ].
] ifFalse:[
  str := super lastErrorString .
].
^ str
%

category: 'Socket Operations'
method:
close

"Closes the secure connection and releases memory used by the receiver's
 SSL connection if it exists.  Then closes the underlying socket connection.

 Returns self if the socket was closed successfully or nil if an error occurs."

self hasSecureConnection
  ifTrue: [ self secureClose ].
^ super close
%

category: 'Examples'
classmethod:
clientExample: logToGciClient usingPort: portNum

| sslClient failBlock bytesRead dataBuffer dataLength data gotEof |

failBlock := [:sock :shouldLog| | err arr |
  err := sock fetchLastIoErrorString .
  err ifNotNil: [GsFile gciLog: err onClient: shouldLog].
  arr := sock class fetchErrorStringArray .
  1 to: arr size do:[:n| GsFile gciLog: (arr at: n) onClient: shouldLog ].
  sock close.
  SocketError signal: err asString.
].

"Setup a normal socket connection first."

"Create an new SSL client socket"
sslClient:= self newClient.

"Now connect to the server to get a normal socket connection."
(sslClient connectTo: portNum) 
  ifTrue:[ GsFile gciLog: 'regular socket connect completed OK.' 
                  onClient: logToGciClient ]
  ifFalse:[ 
    sslClient close .
    sslClient halt: 'Unable to connect to port ' , portNum asString , 
       '. Wait 30 seconds or so, then ensure serverExample is started first.' 
  ].

"uncomment next line to put the socket in blocking mode."
"sslClient makeBlocking."

"Specify the file which contains a list of trusted CA certificates.
 These will be used to validate the certificate presented by the peer
 during the SSL handshake.  The file must be in PEM format."
(GsSecureSocket useCACertificateFileForClients: 
      '$GEMSTONE/examples/openssl/certs/serverCA.pem')
  ifFalse:[GsFile gciLog: 'useCACertificateFileForClients failed'  
                  onClient: logToGciClient
  ].

"Tell SSL that all clients are to drop the connection unless
 the certificate presented by the server is authenticated."
GsSecureSocket enableCertificateVerificationOnClient .

"Set the cipher list for client SSL sockets.
 Use all ciphers except NULL ciphers and anonymous Diffie-Hellman
 and sort by strength."
(GsSecureSocket setClientCipherListFromString: 'ALL:!ADH:@STRENGTH')
  ifFalse:[ GsFile gciLog: 'setClientCipherListFromString: failed'  
                  onClient: logToGciClient
  ].

"Attempt to establish a secure connection to the SSL server..."
sslClient secureConnect
  ifTrue:[ GsFile gciLog: 'Secure connection established'
                  onClient: logToGciClient]
  ifFalse:[ |certError|
    GsFile gciLog: 'secureConnect failed' onClient: logToGciClient.
    certError := GsSecureSocket fetchLastCertificateVerificationErrorForClient .
    certError ifNil: [  
       GsFile gciLog: 'nil result from fetchLastCertificateVerificationErrorForClient' 
              onClient: logToGciClient
        ] ifNotNil: [
        GsFile gciLog: ('Cert error: =>' , certError) onClient: logToGciClient].
  failBlock value: sslClient value: logToGciClient.

  ].

"We have a secure connection to the server if we get here."
GsFile gciLog: ('Current cipher in use is: ', sslClient fetchCipherDescription)
       onClient: logToGciClient.

"Read length from the client"
GsFile gciLog: 'Starting read' onClient: logToGciClient.

dataLength := sslClient read: 8.
(dataLength isNil or: [dataLength size < 8]) 
     ifTrue: [GsFile gciLog: 'read of 8 bytes of length failed' 
				onClient: logToGciClient .
	     		failBlock value: sslClient value: logToGciClient]
     ifFalse: [GsFile gciLog: 'read 8 bytes of length, encoded ', dataLength 
				onClient: logToGciClient].
dataLength := Integer fromHexString: dataLength.
GsFile gciLog: 'Data length should be ', dataLength asString 
	onClient: logToGciClient.

dataBuffer := String new.
gotEof := false .
[(dataBuffer size < (dataLength + 10)) and:[ gotEof not]] whileTrue: [
   bytesRead := sslClient read: 16384 into: dataBuffer startingAt: dataBuffer size + 1.
   bytesRead ifNil:[
     GsFile gciLog: 'read failed' onClient: logToGciClient .
            failBlock value: sslClient  value: logToGciClient
   ] ifNotNil:[
      bytesRead == 0 ifTrue:[ gotEof := true ]
        ifFalse:[ GsFile gciLog: 'Read ', bytesRead asString, ' bytes from peer.'
                  onClient: logToGciClient ].
   ].
].
gotEof ifFalse:[
  GsFile gciLog: 'did not get EOF' onClient: logToGciClient. 
  failBlock value: sslClient value: logToGciClient. 
].
dataBuffer size ~= dataLength ifTrue: [
  GsFile gciLog: 'Read failed ' onClient: logToGciClient.
  failBlock value: sslClient value: logToGciClient.
].
GsFile gciLog: 'Finished reading, read ', dataBuffer size asString, ' bytes from peer' 
	onClient: logToGciClient .

sslClient close.

data := (PassiveObject newWithContents: dataBuffer) activate.
GsFile gciLog: 'Result is ', data class asString, ' of size ', data size asString
     onClient: logToGciClient.
^true
%


category: 'Examples'
classmethod:
serverExample

^ self serverExample: true usingPort: 57785
%

category: 'Examples'
classmethod:
serverExample: logToGciClient usingPort: portNum

| listener sslServer failBlock classFailBlock bigStr result data dataSizeString |

failBlock := [:sock :shouldLog | | err arr |
  err := sock fetchLastIoErrorString .
  err ifNotNil: [GsFile gciLog: err onClient: shouldLog].
  arr := sock class fetchErrorStringArray .
  1 to: arr size do: [:n| GsFile gciLog: (arr at: n) onClient: shouldLog ].
  sock close.
  SocketError signal: err asString.
].

classFailBlock := [:aClass :shouldLog | | arr |
  arr :=  aClass fetchErrorStringArray .
  1 to: arr size do: [:n | GsFile gciLog: (arr at: n) onClient: shouldLog  ].
  SocketError signal: 'Primitive failed' .
].

"Specify the certificate and private key to use for all server connections in
 this gem.  Both must be in PEM format.  Private key and certificate may be
 contained in the same PEM file, as we show here. This private key does not
 need a passphrase, so we use nil as the last argument."

(GsSecureSocket useServerCertificateFile: '$GEMSTONE/examples/openssl/certs/server.pem'
  withPrivateKeyFile: '$GEMSTONE/examples/openssl/certs/server.pem'
  privateKeyPassphrase: nil)
  ifFalse:[ 
    GsFile gciLog: 'useServerCertificateFile failed.' onClient: logToGciClient .
    classFailBlock value: GsSecureSocket value: logToGciClient.
].

"Don't request a certificate from the client.  This is typical."
GsSecureSocket disableCertificateVerificationOnServer .

"Use all ciphers except NULL ciphers and anonymous Diffie-Hellman
 and sort by strength."
(GsSecureSocket setServerCipherListFromString: 'ALL:!ADH:@STRENGTH')
  ifFalse:[ GsFile gciLog: 'setServerCipherListFromString: failed'  
                  onClient: logToGciClient
  ].

"All class methods have been executed, so now we can create the instance.
 Class methods executed AFTER the instance is created will NOT APPLY to
 the instance!"

"Create a new SSL server socket."
listener := self newServer.

"Make it listen for the client."
(listener makeServerAtPort: portNum) ifNil: [ | err |
	err := listener lastErrorString.
	listener close.
	SocketError signal: 'makeServerAtPort failed, ', err asString.
        ].

listener port == portNum ifFalse:[ listener halt: 'bad port number' ] .

GsFile gciLog: 'Waiting for GsSecureSocket clientExample to connect...' 
	onClient: logToGciClient .
[listener readWillNotBlockWithin: 5000] whileFalse: [
	GsFile gciLog: 'waiting...' onClient: logToGciClient
  	].

GsFile gciLog: 'Client connected.' onClient: logToGciClient .

"Do the normal accept to establish a standard socket connection."
"sslServer is our connection to the client."
sslServer := listener accept .
sslServer ifNil: [| err |
        err := listener lastErrorString.
        listener close.
	SocketError signal: 'accept failed, ', err asString.
	].

GsFile gciLog: 'Normal accept completed OK.' onClient: logToGciClient .

"uncomment next to put the socket in blocking mode."
"sslServer makeBlocking."

"sslServer has only a normal connection to the peer.  If will have a secure
  connection if secureAccept succeeds."

sslServer secureAccept
  ifTrue:[ GsFile gciLog: 'Secure connection established.' 
                  onClient: logToGciClient ]
  ifFalse:[ | certError |
    GsFile gciLog: 'secureAccept failed.'  onClient: logToGciClient.
    certError := sslServer class fetchLastCertificateVerificationErrorForServer .
    certError ifNil: [  
        GsFile gciLog: 'nil result from fetchLastCertificateVerificationErrorForServer' 
           onClient: logToGciClient
    ] ifNotNil:[
        GsFile gciLog: ('Cert error: =>' , certError) 
           onClient: logToGciClient.].
    failBlock value: sslServer value: logToGciClient.
].

"If we get here, we have a secure connection to the client."

GsFile gciLog: ('Current cipher in use is: ', sslServer fetchCipherDescription)
       onClient: logToGciClient.

"Build data to send"
data := Array new.
4096 timesRepeat: [ data add: (String withAll: '0123456789ABCDEF') ].
bigStr := String new .
PassiveObject passivate: data toStream: (WriteStream on: bigStr).
dataSizeString := bigStr size asHexStringWithLength: 8.
dataSizeString size > 8 ifTrue: 
    [UserDefinedError signal: 'Protocol Error, attempt to send more than 4294967295 bytes'].

GsFile gciLog: 'Sending data length to client: ', bigStr size asString, 
    ' bytes, encoded as ', dataSizeString onClient: logToGciClient.

result := sslServer write: dataSizeString.
result == dataSizeString size
  ifTrue: [ GsFile gciLog: 'Finished sending length to peer' 
		onClient: logToGciClient ]
  ifFalse: [ GsFile gciLog: 'write of length bytes failed, result was ', result asString
                onClient: logToGciClient .
            failBlock value: sslServer value: logToGciClient.
		].

GsFile gciLog: 'Sending ', bigStr size asString, ' bytes to client' 
       onClient: logToGciClient.
result := sslServer write: bigStr .
result == bigStr size
  ifTrue:[ GsFile gciLog: ('Finished sending ', bigStr size asString, ' bytes to peer')
                   onClient: logToGciClient ]
  ifFalse:[ GsFile gciLog: ('write failed, result was ', result asString) 
                   onClient: logToGciClient .
            failBlock value: sslServer value: logToGciClient.
            ].
"close method will close the SSL connection and underlying socket."
sslServer close.
listener close .
^ true
%

category: 'Examples'
classmethod:
clientExample

^ self clientExample: true usingPort: 57785
%

category: 'Error Handling'
classmethod:
fetchErrorStringArray

"Returns an Array of error strings generated by the OpenSSL package.
 The errors returned are cleared from the SSL error queue.  The array
 is ordered from oldest to newest error."

^ self _zeroArgSslPrim: 6
%

category: 'Certificate Authorities'
classmethod: 
useCACertificateFileForClients: aFileNameString

"Attempt to load given CA Certificate file which must be in PEM format.
 The file may contain more than one certificate.  If the file is loaded
 successfully, it will be used by SSL clients to validate all certificates
 presented by SSL servers during the SSL handshake.

 The certificates loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 Has no effect on instances created before the method was called.

 Also enables certificate verification for client sockets as if 
 the #enableCertificateVerificationOnClient method had been called
 immediately following this method.

 Returns true on success or false if an error occurs.  Raises an 
 exception if the file does not exist."

^ self _oneArgSslPrim: 1 with: aFileNameString
%

category: 'Certificate Authorities'
classmethod:
useCACertificateFileForServers: aFileNameString

"Attempt to load given CA Certificate file which must be in PEM format.
 The file may contain more than one certificate.  The certificates
 are considered to be trusted and will be used to validate certificates
 presented by SSL clients.  Note that authentication of clients is disabled
 by default and must be enabled in order for this method to have any effect.

 The certificates loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 Has no effect on instances created before the method was called.

 Also enables certificate verification for server sockets as if 
 the #enableCertificateVerificationOnServer method had been called
 immediately following this method.

 Returns true on success or false if an error occurs.  Raises an 
 exception if the file does not exist."

^ self _oneArgSslPrim: 2 with: aFileNameString
%

category: 'Peer Authentication'
classmethod:
enableCertificateVerificationOnServer
"Directs server sockets to request and verify a certificate from the client.
 The connection will fail if the client does not provide a certificate or
 the certificate is found to be invalid.

 The effect is valid for the current session only; 
 the affected SSL state is not committed to the repository.

 Has no effect on instances created before the method was called.

 By default, servers do not request certificates from clients.

 Returns the receiver."

^ self _zeroArgSslPrim: 11
%

category: 'Peer Authentication'
classmethod:
disableCertificateVerificationOnServer
"Directs server sockets to not request a certificate from the client.
 This is the default mode for server sockets.

 The effect is valid for the current session only; 
 the affected SSL state is not committed to the repository.

 Has no effect on instances created before the method was called.

 Returns the receiver."

^ self _zeroArgSslPrim: 10
%

category: 'Peer Authentication'
classmethod:
enableCertificateVerificationOnClient
"Directs client sockets to verify the certificate from the server.
 The connection will fail if the server does not provide a valid
 certificate to the client.

 The client always requests a certificate from the server.

 This is the default mode for client sockets.

 The effect is valid for the current session only; 
 the affected SSL state is not committed to the repository.

 Has no effect on instances created before the method was called.

 Returns the receiver."

^ self _zeroArgSslPrim: 9
%

category: 'Peer Authentication'
classmethod:
disableCertificateVerificationOnClient
"Directs new client sockets ignore invalid server certificates.  The connection
 will not fail if the server certificate is found to be invalid.

 The client always requests a certificate from the server.  By default,
 the connection will fail if the certificate from the server is invalid.

 The effect is valid for the current session only; 
 the affected SSL state is not committed to the repository.

 Has no effect on instances created before the method was called.

 Returns the receiver."

^ self _zeroArgSslPrim: 8
%

category: 'Certificates and Keys'
classmethod:
useServerCertificateFile: certFile withPrivateKeyFile: keyFile privateKeyPassphrase: pass

"Specifies the certificate file and private key file to be used by all
 server connections.  The private key must match the certificate.

 Both files must be in PEM format.  

 The certificate file may contain a certificate chain or a single certificate.

 If the private key requires a passphrase, it is to be specified as String
 passed in using the pass argument.  If the private key does not require a
 passphrase, then the pass argument is expected to be nil.

 Both file arguments must be instances of String.

 The certificates loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 Returns true on success or false if an error occurs.
 Raises an exception if either file does not exist."

^ self _threeArgSslPrim: 6 with: certFile with: keyFile with: pass
%

category: 'Certificates and Keys'
classmethod:
useClientCertificateFile: certFile withPrivateKeyFile: keyFile privateKeyPassphrase: pass

"Specifies the certificate file and private key file to be used by all
 client connections.  The private key must match the certificate.

 The certificates loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 Both files must be in PEM format.  

 The certificate file may contain a certificate chain or a single certificate.

 If the private key requires a passphrase, it is to be specified as String
 passed in using the pass argument.  If the private key does not require a
 passphrase, then the pass argument is expected to be nil.

 Both file arguments must be instances of String.

 Returns true on success or false if an error occurs.
 Raises an exception if either file does not exist."

^ self _threeArgSslPrim: 7 with: certFile with: keyFile with: pass
%

category: 'Certificates and Keys'
method:
useCertificateFile: certFile withPrivateKeyFile: keyFile privateKeyPassphrase: pass

"Specifies the certificate file and private key file to be used by
 the receiver only.  The private key must match the certificate.

 This method must be run before the #secureAccept (server case)
 or the #secureConnect (client case) methods.
 The certificates loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 If the private key requires a passphrase, it is to be specified as String
 passed in using the pass argument.  If the private key does not require a
 passphrase, then the pass argument is expected to be nil.

 Both files must be in PEM format.  

 Both arguments must be instances of String.

 Returns true on success or false if an error occurs.
 Raises an exception if either file does not exist."

^ self _threeArgSslPrim: 8 with: certFile with: keyFile with: pass
%

category: 'Error Handling'
classmethod:
clearErrorQueue

"Clears all errors from the error queue for all GsSecureSocket instances.
 Returns the receiver."

^ self _zeroArgSslPrim: 7
%

category: 'Error Handling'
classmethod:
fetchLastCertificateVerificationErrorForServer

"Fetches and clears a string representing the last certificate verification
 error logged by a server SSL socket.  Returns nil if no such error has 
 occurred."

^ self _zeroArgSslPrim: 14
%

category: 'Error Handling'
classmethod:
fetchLastCertificateVerificationErrorForClient

"Fetches and clears a string representing the last certificate verification
 error logged by a client SSL socket.  Returns nil if no such error has
 occurred."

^ self _zeroArgSslPrim: 15
%

category: 'Querying'
method:
fetchCipherDescription

"If the receiver has a secure connection with its peer, this method answers a string which
 describes the cipher currently in use.  Otherwise returns nil to indicate
 the receiver does not have a secure connection with its peer."

^ self _zeroArgSslPrim: 16
%

category: 'Certificates and Keys'
classmethod:
useServerCertificate: aString withPrivateKey: anotherString privateKeyPassphrase: passPhrase

"Specifies the certificate string and private key string to be used by all
 server connections.  The private key must match the certificate.

 The certificates loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 Both strings must be in PEM format.  If the private key requires a passphrase,
 it is specified in the passPhrase argument as a String.  If no passphrase is 
 required, then the passPhrase argument is expected to be nil.

 Both strings must exactly match the contents of the corresponding certificate files
 (including white-space characters) or the strings will not be accepted.

 Returns true on success or false if an error occurs."

^ self _threeArgSslPrim: 3 with: aString with: anotherString with: passPhrase
%

category: 'Certificates and Keys'
classmethod:
useClientCertificate: aString withPrivateKey: anotherString privateKeyPassphrase: passPhrase

"Specifies the certificate string and private key string to be used by all
 client connections.  The private key must match the certificate.

 The certificates loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 Both strings must be in PEM format.  If the private key requires a passphrase,
 it is specified in the passPhrase argument as a String.  If no passphrase is 
 required, then the passPhrase argument is expected to be nil.

 Both strings must exactly match the contents of the corresponding certificate files
 (including white-space characters) or the strings will not be accepted.

 Returns true on success or false if an error occurs."

^ self _threeArgSslPrim: 4 with: aString with: anotherString with: passPhrase
%

category: 'Certificates and Keys'
method:
useCertificate: aString withPrivateKey: anotherString privateKeyPassphrase: passPhrase

"Specifies the certificate and private key to be used by
 the receiver only.  The private key must match the certificate.

 The certificates loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 Both strings must be in PEM format.  If the private key requires a passphrase,
 it is specified in the passPhrase argument as a String.  If no passphrase is 
 required, then the passPhrase argument is expected to be nil.

 Both strings must exactly match the contents of the corresponding certificate files
 (including white-space characters) or the strings will not be accepted.

 Returns true on success or false if an error occurs."

^ self _threeArgSslPrim: 5 with: aString with: anotherString with: passPhrase
%

category: 'Ciphers'
classmethod:
setClientCipherListFromString: aString

"Specify the cipher list to be used by all client connections.  aString must be an 
 instance of String in the format described in the man page for ciphers(1). See
   http://www.openssl.org/docs/apps/ciphers.html

 The ciphers loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 Returns true if one or more of the specified ciphers are usable, false if no 
 specified ciphers are usable."

^ self _oneArgSslPrim: 3 with: aString
%

category: 'Ciphers'
classmethod:
setServerCipherListFromString: aString

"Specify the cipher list to be used by all server connections.  aString must be an 
 instance of String in the format described in the man page for ciphers(1).  See
   http://www.openssl.org/docs/apps/ciphers.html

 The ciphers loaded into the internal SSL state are valid for the
 current session only; that SSL state is not committed to the repository.

 Returns true if one or more of the specified ciphers are usable, false if no 
 specified ciphers are usable."

^ self _oneArgSslPrim: 4 with: aString
%

category: 'Ciphers'
method:
setCipherListFromString: aString

"Specify the cipher list to be used by the receiver only.  aString must be an 
 instance of String in the format described in the man page for ciphers(1).  See
   http://www.openssl.org/docs/apps/ciphers.html

 Must be executed before the receiver is in a connected state (i.e., before
 the #secureConnect or #secureAccept method has been executed).

 Returns true if one or more of the specified ciphers are usable, false if no 
 specified ciphers are usable.  Returns nil if the operation had no effect 
 because the receiver is already in a connected state."

^ self _oneArgSslPrim: 5 with: aString
%

category: 'Examples'
classmethod:
clientExample2

"This version uses GsSecureSocket instance methods for setup rather
 than class methods."

| sslClient failBlock bytesRead dataBuffer dataLength data portNum 
  logToGciClient |

portNum := 57785.
logToGciClient := true.
failBlock := [:sock :shouldLog| | err arr |
  err := sock fetchLastIoErrorString .
  err ifNotNil: [GsFile gciLog: err onClient: shouldLog].
  arr := sock class fetchErrorStringArray .
  1 to: arr size do:[:n| GsFile gciLog: (arr at: n) onClient: shouldLog ].
  sock close.
  SocketError signal: err asString.
].

"Specify the file which contains a list of trusted CA certificates.
 These will be used to validate the certificate presented by the peer
 during the SSL handshake.  The file must be in PEM format.

 Currently, there is no easy way to specify the CA list from a String.

 This must be done before creating the instance of GsSecureSocket! "

(GsSecureSocket useCACertificateFileForClients: 
      '$GEMSTONE/examples/openssl/certs/serverCA.pem')
  ifFalse:[GsFile gciLog: 'useCACertificateFileForClients failed'  
                  onClient: logToGciClient
  ].

"Setup a normal socket connection first."

"Create an new SSL client socket"
sslClient:= self newClient.

"Now connect to the server to get a normal socket connection."
(sslClient connectTo: portNum) 
  ifTrue:[ GsFile gciLog: 'regular socket connect completed OK.' 
                  onClient: logToGciClient ]
  ifFalse:[ 
    sslClient close .
    sslClient halt: 'Unable to connect to port ' , portNum asString , 
       '. Wait 30 seconds or so, then ensure serverExample is started first.' 
  ].

"uncomment next line to put the socket in blocking mode."
"sslClient makeBlocking."

"Tell SSL that this SSL client is to drop the connection unless
 the certificate presented by the server is authenticated."
sslClient enableCertificateVerification .

"Set the cipher list for this SSL socket only.
 Use all ciphers except NULL ciphers and anonymous Diffie-Hellman
 and sort by strength."
(sslClient setCipherListFromString: 'ALL:!ADH:@STRENGTH')
  ifFalse:[ GsFile gciLog: 'setCipherListFromString: failed'  
                  onClient: logToGciClient
  ].

"Attempt to establish a secure connection to the SSL server..."
sslClient secureConnect
  ifTrue:[ GsFile gciLog: 'Secure connection established'
                  onClient: logToGciClient]
  ifFalse:[ |certError|
    GsFile gciLog: 'secureConnect failed' onClient: logToGciClient.
    certError := GsSecureSocket fetchLastCertificateVerificationErrorForClient .
    certError ifNil: [  
       GsFile gciLog: 'nil result from fetchLastCertificateVerificationErrorForClient' 
              onClient: logToGciClient
        ] ifNotNil: [
        GsFile gciLog: ('Cert error: =>' , certError) onClient: logToGciClient].
  failBlock value: sslClient value: logToGciClient.

  ].

"We have a secure connection to the server if we get here."
GsFile gciLog: ('Current cipher in use is: ', sslClient fetchCipherDescription)
       onClient: logToGciClient.

"Read length from the client"
GsFile gciLog: 'Starting read' onClient: logToGciClient.

dataLength := sslClient read: 8.
(dataLength isNil or: [dataLength size < 8]) 
     ifTrue: [GsFile gciLog: 'read of 8 bytes of length failed' 
				onClient: logToGciClient .
	     		failBlock value: sslClient value: logToGciClient]
     ifFalse: [GsFile gciLog: 'read 8 bytes of length, encoded ', dataLength 
				onClient: logToGciClient].
dataLength := Integer fromHexString: dataLength.
GsFile gciLog: 'Data length should be ', dataLength asString 
	onClient: logToGciClient.

dataBuffer := String new.
[dataBuffer size < dataLength ] 
   whileTrue:
      [bytesRead := sslClient read: 16384 into: dataBuffer startingAt: dataBuffer size + 1.
      (bytesRead isNil or: [bytesRead = 0])
	  ifTrue: [
	    GsFile gciLog: 'read failed' onClient: logToGciClient .
            failBlock value: sslClient  value: logToGciClient
            ]
	  ifFalse: [
	    GsFile gciLog: 'Read ', bytesRead asString, ' bytes from peer.' 
                  onClient: logToGciClient 
            ]
     ].
dataBuffer size ~= dataLength 
    ifTrue: [GsFile gciLog: 'Read failed ' onClient: logToGciClient. 
            failBlock value: sslClient value: logToGciClient.
	    ].

GsFile gciLog: 'Finished reading, read ', dataBuffer size asString, ' bytes from peer' 
	onClient: logToGciClient .

sslClient close.

data := (PassiveObject newWithContents: dataBuffer) activate.
GsFile gciLog: 'Result is ', data class asString, ' of size ', data size asString
     onClient: logToGciClient.
^true
%

category: 'Examples'
classmethod:
serverExample2

"This version uses GsSecureSocket instance methods for setup rather 
 than class methods."

| listener sslServer failBlock classFailBlock bigStr result data dataSizeString
  logToGciClient portNum |

portNum := 57785.
logToGciClient := true.

failBlock := [:sock :shouldLog | | err arr |
  err := sock fetchLastIoErrorString .
  err ifNotNil: [GsFile gciLog: err onClient: shouldLog].
  arr := sock class fetchErrorStringArray .
  1 to: arr size do: [:n| GsFile gciLog: (arr at: n) onClient: shouldLog ].
  sock close.
  SocketError signal: err asString.
].

classFailBlock := [:aClass :shouldLog | | arr |
  arr :=  aClass fetchErrorStringArray .
  1 to: arr size do: [:n | GsFile gciLog: (arr at: n) onClient: shouldLog  ].
  SocketError signal: 'Primitive failed' .
].

"Create a new SSL server socket."
listener := self newServer.

"Make it listen for the client."
(listener makeServerAtPort: portNum) ifNil: [ | err |
	err := listener lastErrorString.
	listener close.
	SocketError signal: 'makeServerAtPort failed, ', err asString.
        ].

listener port == portNum ifFalse:[ listener halt: 'bad port number' ] .

GsFile gciLog: 'Waiting for GsSecureSocket clientExample to connect...' 
	onClient: logToGciClient .
[listener readWillNotBlockWithin: 5000] whileFalse: [
	GsFile gciLog: 'waiting...' onClient: logToGciClient
  	].

GsFile gciLog: 'Client connected.' onClient: logToGciClient .

"Do the normal accept to establish a standard socket connection."
"sslServer is our connection to the client."
sslServer := listener accept .
sslServer ifNil: [| err |
        err := listener lastErrorString.
        listener close.
	SocketError signal: 'accept failed, ', err asString.
	].

GsFile gciLog: 'Normal accept completed OK.' onClient: logToGciClient .

"uncomment next to put the socket in blocking mode."
"sslServer makeBlocking."

"Don't request a certificate from the client.  This is typical."
sslServer disableCertificateVerification.

"sslServer has only a normal connection to the peer.  If will have a secure
  connection if secureAccept succeeds."

"Example of using an encrypted private key that requires a passphrase."
(sslServer useCertificateFile: '$GEMSTONE/examples/openssl/certs/server.pem'
  withPrivateKeyFile: '$GEMSTONE/examples/openssl/certs/serverkey.pem'
  privateKeyPassphrase: 'SERVERKEY' )
  ifFalse:[ 
    GsFile gciLog: 'useCertificateFile failed.' onClient: logToGciClient .
    classFailBlock value: GsSecureSocket value: logToGciClient.
].

"Use all ciphers except NULL ciphers and anonymous Diffie-Hellman
 and sort by strength."
(sslServer setCipherListFromString: 'ALL:!ADH:@STRENGTH')
  ifFalse:[ GsFile gciLog: 'setCipherListFromString: failed'  
                  onClient: logToGciClient
  ].

sslServer secureAccept
  ifTrue:[ GsFile gciLog: 'Secure connection established.' 
                  onClient: logToGciClient ]
  ifFalse:[ | certError |
    GsFile gciLog: 'secureAccept failed.'  onClient: logToGciClient.
    certError := sslServer class fetchLastCertificateVerificationErrorForServer .
    certError ifNil: [  
        GsFile gciLog: 'nil result from fetchLastCertificateVerificationErrorForServer' 
           onClient: logToGciClient
    ] ifNotNil:[
        GsFile gciLog: ('Cert error: =>' , certError) 
           onClient: logToGciClient.].
    failBlock value: sslServer value: logToGciClient.
].

"If we get here, we have a secure connection to the client."

GsFile gciLog: ('Current cipher in use is: ', sslServer fetchCipherDescription)
       onClient: logToGciClient.

"Build data to send"
data := Array new.
4096 timesRepeat: [ data add: (String withAll: '0123456789ABCDEF') ].
bigStr := String new .
PassiveObject passivate: data toStream: (WriteStream on: bigStr).
dataSizeString := bigStr size asHexStringWithLength: 8.
dataSizeString size > 8 ifTrue: 
    [UserDefinedError signal: 'Protocol Error, attempt to send more than 4294967295 bytes'].

GsFile gciLog: 'Sending data length to client: ', bigStr size asString, 
    ' bytes, encoded as ', dataSizeString onClient: logToGciClient.

result := sslServer write: dataSizeString.
result == dataSizeString size
  ifTrue: [ GsFile gciLog: 'Finished sending length to peer' 
		onClient: logToGciClient ]
  ifFalse: [ GsFile gciLog: 'write of length bytes failed, result was ', result asString
                onClient: logToGciClient .
            failBlock value: sslServer value: logToGciClient.
		].

GsFile gciLog: 'Sending ', bigStr size asString, ' bytes to client' 
       onClient: logToGciClient.
result := sslServer write: bigStr .
result == bigStr size
  ifTrue:[ GsFile gciLog: ('Finished sending ', bigStr size asString, ' bytes to peer')
                   onClient: logToGciClient ]
  ifFalse:[ GsFile gciLog: ('write failed, result was ', result asString) 
                   onClient: logToGciClient .
            failBlock value: sslServer value: logToGciClient.
            ].

"close method will close the SSL connection and underlying socket."
sslServer close.
listener close .
^ true
%

category: 'Peer Authentication'
method:
disableCertificateVerification
"Directs the receiver to not request a certificate from its peer.
 This is the default mode for server sockets.

 Returns true on success or nil on error."

^ self _zeroArgSslPrim: 17
%

category: 'Peer Authentication'
method:
enableCertificateVerification
"Directs the receiver to request and verify a certificate from its peer.
 This is the default mode for client sockets.

 Returns true on success or nil on error."

^ self _zeroArgSslPrim: 18
%

! Feature 44331
category: 'Querying'
classmethod:
sslLibraryVersionString

"Answer an instance of String which describes the version of OpenSSL library loaded by this session."

^ self _zeroArgSslPrim: 19
%

category: 'Private'
method:
_maxReadWaits
  "The wait loop in GsSocket>>read:into:startingAt can
   get repeated read-ready of the underlying socket
   followed by SSL read apparently peeking
   on the socket and returning EWOULDBLOCK because there is not
   yet enough data to do a decryption cycle."
  ^ SmallInteger maximumValue
%

! fix 46174
category: 'Testing'
method: 
readWillNotBlockWithin: msToWait

"Returns true if the socket is ready to receive input without blocking within
 msToWait milliseconds from the time that this method is called.  Returns false
 if it is not ready after msToWait milliseconds.  Returns nil if an error
 occurs.

 If msToWait is 0, then this method reports the current readiness of the
 receiver.  If msToWait is -1, then this method never returns false, but waits
 until the receiver is ready to receive input without blocking, and then returns
 true.

 The receiver must already be connected for this method to work properly.  If it
 is not connected, then the value that this method returns is indeterminate.
 Use the peerName method to determine if the receiver is connected.

 Call this method to prevent subsequent read or accept operations from hanging.
 If it returns true for a connected socket, then the input operation will not
 hang.  However, a return value of true is no guarantee that the operation
 itself will succeed."

 self _peek ifNotNil:[ ^ true ].
 ^ super readWillNotBlockWithin: msToWait
%

category: 'Testing'
method:
readWillNotBlock

"Returns true if the socket is currently ready to receive input without
 blocking.  Returns false if it is not currently ready.  Returns nil if an error
 occurs.

 The receiver must already be connected for this method to work properly.  If it
 is not connected, then the value that this method returns is indeterminate.
 Use the peerName method to determine if the receiver is connected.

 Call this method to prevent subsequent read or accept operations from hanging.
 If it returns true for a connected socket, then the input operation will not
 hang.  However, a return value of true is no guarantee that the operation
 itself will succeed."

 self _peek ifNotNil:[ ^ true ].
 ^ super readWillNotBlock
%

category: 'Private'
method:
_peek
  "Returns a SmallInteger in the range 0..255 if there is a byte available 
   from the underlying SSL buffer for the socket, otherwise returns nil."

  ^ self _zeroArgSslPrim: 20
%

category: 'Querying'
method:
certificateVerificationEnabled

"Answer true if certificate verification is enabled on the receiver.
 Otherwise answer false."

^ self _zeroArgSslPrim: 21
%

category: 'Querying'
classmethod:
certificateVerificationEnabledOnClient

"Answer true if certificate verification is enabled for client sockets.
 Otherwise answer false."

^ self _zeroArgSslPrim: 22
%

category: 'Querying'
classmethod:
certificateVerificationEnabledOnServer

"Answer true if certificate verification is enabled for server sockets.
 Otherwise answer false."

^ self _zeroArgSslPrim: 23
%

category: 'Querying'
method:
fetchCertificateVerificationOptions

"Answers an Array of Symbols which represent the certificate verification 
 options used by the receiver.  Certificate verification options are only
 supported by server sockets.  Client sockets always return an empty Array.

 The supported server options are:

 #SSL_VERIFY_FAIL_IF_NO_PEER_CERT
    if the client did not return a certificate, the TLS/SSL handshake is 
    immediately terminated with a 'handshake failure' alert.

 #SSL_VERIFY_CLIENT_ONCE
    only request a client certificate on the initial TLS/SSL handshake. 
    Do not ask for a client certificate again in case of a renegotiation.

 Refer to the OpenSSL documentation for more information about these options."

^ self _zeroArgSslPrim: 24
%


category: 'Querying'
classmethod:
fetchCertificateVerificationOptionsForServer

"Answers an Array of Symbols which represent the certificate verification 
 options used by server sockets.

 The supported server options are:

 #SSL_VERIFY_FAIL_IF_NO_PEER_CERT
    if the client did not return a certificate, the TLS/SSL handshake is 
    immediately terminated with a 'handshake failure' alert.

 #SSL_VERIFY_CLIENT_ONCE
    only request a client certificate on the initial TLS/SSL handshake. 
    Do not ask for a client certificate again in case of a renegotiation.

 Refer to the OpenSSL documentation for more information about these options."

^ self _zeroArgSslPrim: 25
%

category: 'Peer Authentication'
classmethod:
setCertificateVerificationOptionsForServer: anArray

"Sets the certificate verification options for server sockets using an Array of
 Symbols.  The supported server options are:

 #SSL_VERIFY_FAIL_IF_NO_PEER_CERT
    if the client did not return a certificate, the TLS/SSL handshake is 
    immediately terminated with a 'handshake failure' alert.

 #SSL_VERIFY_CLIENT_ONCE
    only request a client certificate on the initial TLS/SSL handshake. 
    Do not ask for a client certificate again in case of a renegotiation.

 Refer to the OpenSSL documentation for more information about these options.

 If anArray is empty then any previously set options are cleared.

 Has no effect on instances created before the method was called.

 Raises an error if the array contains any elements besides the above symbols.
 
 Certificate verification for server sockets must be enabled before executing this
 method, otherwise an error is raised.  Use the #enableCertificateVerificationOnServer
 method to enable certification verification.

 Returns the receiver."

^ self _oneArgSslPrim: 6 with: anArray
%

category: 'Peer Authentication'
method:
setCertificateVerificationOptions: anArray

"Sets the certificate verification options for the receiver using an Array of
 Symbols.  

 The supported server socket options are:

 #SSL_VERIFY_FAIL_IF_NO_PEER_CERT
    if the client did not return a certificate, the TLS/SSL handshake is 
    immediately terminated with a 'handshake failure' alert.

 #SSL_VERIFY_CLIENT_ONCE
    only request a client certificate on the initial TLS/SSL handshake. 
    Do not ask for a client certificate again in case of a renegotiation.

 Refer to the OpenSSL documentation for more information about these options.

 Raises an error if the array contains any elements besides the above symbols.

 Currently there are no supported options for client sockets. Using this method
 with a client socket raises an error.

 If anArray is empty then any previously set options are cleared.

 The receiver must enable certificate verification before this method is
 executed, otherwise an error is raised.  Use the #enableCertificateVerification 
 method to enable certificate verification.

 This method must be used before the receiver attempts a connection with its peer.
 Using this method with a connected socket raises an error. 

 Returns the receiver."

^ self _oneArgSslPrim: 7 with: anArray
%

category: 'Certificate Authorities'
classmethod: 
useCACertificateDirectoryForClients: aDirectoryString

"Specifies a directory where trusted certificate authority (CA) certificates in
 PEM format are located. The certificates in this directory will be used to 
 authenticate certificates provided by servers during the SSL handshake. The 
 directory may contain more than one certificate. 

 Certificates are loaded into the internal SSL state which is valid for the
 current session only; the SSL state is not committed to the repository.

 Has no effect on instances created before the method was called or server
 instances of GsSecureSocket.

 If successful, this method also enables certificate verification for client
 sockets as if the #enableCertificateVerificationOnClient method had been called
 immediately following this method.

 Returns true on success or false if an error occurs.  Raises an 
 exception if the directory does not exist."

^ self _oneArgSslPrim: 8 with: aDirectoryString
%

category: 'Certificate Authorities'
classmethod:
useCACertificateDirectoryForServers: aDirectoryString

"Specifies a directory where trusted certificate authority (CA) certificates in
 PEM format are located. The certificates in this directory will be used to 
 authenticate certificates provided by client during the SSL handshake. The 
 directory may contain more than one certificate. 

 Certificates are loaded into the internal SSL state which is valid for the
 current session only; the SSL state is not committed to the repository.

 Has no effect on instances created before the method was called or client
 instances of GsSecureSocket.

 If successful, this method also enables certificate verification for server
 sockets as if the #enableCertificateVerificationOnServer method had been called
 immediately following this method.

 Returns true on success or false if an error occurs.  Raises an 
 exception if the directory does not exist."

^ self _oneArgSslPrim: 9 with: aDirectoryString
%

category: 'Examples'
classmethod:
httpsClientExampleForHost: hostName certificateDirectory: aDirectory

"Connect to an https web server on port 443 at the given host and
 perform a simple GET request.  Full verification of the server certificate
 is performed.  Self-signed certificates will fail verification."

| sslClient failBlock portNum logToGciClient request bytesSent
  responseString |

portNum := 443. "https port"
logToGciClient := true.
failBlock := [:sock :shouldLog| | err arr |
  err := sock fetchLastIoErrorString .
  err ifNotNil: [GsFile gciLog: err onClient: shouldLog].
  arr := sock class fetchErrorStringArray .
  1 to: arr size do:[:n| GsFile gciLog: (arr at: n) onClient: shouldLog ].
  sock close.
  SocketError signal: err asString.
].

"Specify the directory which contains a list of trusted CA certificates.
 These will be used to validate the certificate presented by the peer
 during the SSL handshake.  The files in the directory must be in PEM
 format and the file names must be the hash value of the CA subject name.
 See https://www.openssl.org/docs/man1.0.2/ssl/SSL_CTX_load_verify_locations.html
 for more information.

 The directory must be specified before creating the instance of GsSecureSocket.

 On Ubuntu Linux, the certificate directory is /etc/ssl/certs  "

(GsSecureSocket useCACertificateDirectoryForClients: aDirectory)
  ifFalse:[GsFile gciLog: 'useCACertificateDirectoryForClients: ', 
                  aDirectory, ' failed'
                  onClient: logToGciClient
].

"Create an new SSL client socket"
sslClient:= GsSecureSocket newClient.

"Now connect to the server to get a normal socket connection."
(sslClient connectTo: portNum on: hostName)
  ifTrue:[ GsFile gciLog: 'regular socket connect completed OK.'
                  onClient: logToGciClient ]
  ifFalse:[
    sslClient close .
    sslClient halt: 'Unable to connect to port ' , portNum asString ,
       ' on host ', hostName
  ].

"Put the socket in blocking mode."
sslClient makeBlocking.

"Tell SSL that this SSL client is to drop the connection unless
 the certificate presented by the server is authenticated."
sslClient enableCertificateVerification .

"Set the cipher list for this SSL socket only.
 Use all ciphers except NULL ciphers and anonymous Diffie-Hellman
 and sort by strength."
(sslClient setCipherListFromString: 'ALL:!ADH:@STRENGTH')
  ifFalse:[ GsFile gciLog: 'setCipherListFromString: failed'
                  onClient: logToGciClient
].

"Attempt to establish a secure connection to the SSL server..."
sslClient secureConnect
  ifTrue:[ GsFile gciLog: 'Secure connection established'
                  onClient: logToGciClient]
  ifFalse:[ |certError|
    GsFile gciLog: 'secureConnect failed' onClient: logToGciClient.
    certError := GsSecureSocket fetchLastCertificateVerificationErrorForClient .
    certError ifNil: [
       GsFile gciLog: 'nil result from fetchLastCertificateVerificationErrorForClient'
              onClient: logToGciClient
        ] ifNotNil: [
        GsFile gciLog: ('Cert error: =>' , certError) onClient: logToGciClient].
  failBlock value: sslClient value: logToGciClient.
].

"We have a secure connection to the server if we get here."

"Get the cipher used and print it"
GsFile gciLog: ('Current cipher in use is: ', sslClient fetchCipherDescription)
       onClient: logToGciClient.

"Build a simple http 1.1 GET request string.  Terminate with a blank line"
request := String withAll: 'GET / HTTP/1.1'.
request add: Character cr ;
        add: Character lf ;
	addAll: 'Host: ';
	addAll: hostName ;
        add: Character cr ;
        add: Character lf ;
        add: Character cr ;
	add: Character lf.

GsFile gciLog: 'Sending a ', request size asString, ' byte request to client: '
     onClient: logToGciClient.
GsFile gciLog: request onClient: logToGciClient.

"Send the GET request to the web server"
bytesSent := sslClient write: request.
bytesSent == request size
  ifTrue: [ GsFile gciLog: 'Finished sending request to peer' 
		onClient: logToGciClient ]
  ifFalse: [ GsFile gciLog: 'write of request failed, result was ', bytesSent asString,
  ' expected ', request size asString
                onClient: logToGciClient .
            failBlock value: sslClient value: logToGciClient.
].

"Get the response from the web server and print it"
GsFile gciLog: 'Waiting for response from server...' onClient: logToGciClient.
responseString := sslClient read: 2048 .
GsFile gciLog: 'Finished reading ', responseString size asString, ' bytes from server.'
		onClient: logToGciClient.
GsFile gciLog: responseString onClient: logToGciClient.

"All done! Close the connection and return"
sslClient close.
^true
%
              
category: 'Examples'
classmethod:
httpsClientExample

"Connect to the google https web server on port 443 at the given host and
 perform a simple GET request.  Full verification of the server certificate
 is performed.  Returns true on success"

^ self httpsClientExampleForHost: 'www.google.com'
       certificateDirectory: '/etc/ssl/certs'
%

! Request 47280
category: 'Instance Creation'
classmethod:
newServerFromGsSocket: aGsSocket

"Creates a new instance and initializes it to act as an SSL server socket.
 Any existing (non-SSL) connection owned by aGsSocket is transferred to the
 new instance and aGsSocket is effectively closed. No SSL connection is made."

^ super new initializeAsServerFromGsSocket: aGsSocket
%

category: 'Instance Creation'
classmethod:
newClientFromGsSocket: aGsSocket

"Creates a new instance and initializes it to act as an SSL client socket.
 Any existing (non-SSL) connection owned by aGsSocket is transferred to the
 new instance and aGsSocket is effectively closed. No SSL connection is made."

^ super new initializeAsClientFromGsSocket: aGsSocket
%

category: 'Instance Creation'
classmethod:
newServerIpv6FromGsSocket: aGsSocket

"Creates a new IPv6 instance and initializes it to act as an SSL server socket.
 Any existing (non-SSL) connection owned by a GsSocket is transferred to the
 new instance and aGsSocket is effectively closed. No SSL connection is made."

^ super newIpv6 initializeAsServerFromGsSocket: aGsSocket
%

category: 'Instance Creation'
classmethod:
newClientIpv6FromGsSocket: aGsSocket

"Creates a new IPv6 instance and initializes it to act as an SSL client socket.
 Any existing (non-SSL) connection owned by a GsSocket is transferred to the
 new instance and aGsSocket is effectively closed. No SSL connection is made."

^ super newIpv6 initializeAsClientFromGsSocket: aGsSocket
%

category: 'Initialization'
method:
initializeAsServerFromGsSocket: aGsSocket

"Returns the receiver on success"
^ self _oneArgSslPrim: 10 with: aGsSocket
%

category: 'Initialization'
method:
initializeAsClientFromGsSocket: aGsSocket

"Returns the receiver on success"
^ self _oneArgSslPrim: 11 with: aGsSocket
%

category: 'Examples'
classmethod: GsSecureSocket
startTlsServerExample: logToGciClient usingPort: portNum address: listeningAddress

"logToGciClient is a Boolean, true if GsFile-based logging is to be done to
 the RPC Gci client, false if to the server process's log file.  Use false if
 executing via topaz 'nbrun'.

 Creates a GsSocket, binds it to port portNum, and waits for a connection.
 When a connection is established, waits for unencrypted data from the client.
 Sends OK back to the client to indicate unencrypted data was received.
 If the unencrypted data contains the string STARTTLS, then a GsSecureSocket
 is created from the GsSocket and secureAccept: is called to negotiate an
 encrypted connection. Once the encrypted connection is established, the 
 cipher in use is printed and the connection is closed.

 All strings sent and received are expected to be NULL-terminated.

 You will need two GemStone sessions running from two independent
 interface processes to run both this and the startTlsClientExample.
   
 Warning: This method will cause your current session to hang until a 
 connection is established."

| server client numWritten errStr sleepTime starttls resultString
  chunk sslServer classFailBlock failBlock dataString |

server := GsSocket new.
(server makeServer: 5 atPort: portNum atAddress: listeningAddress) ifNil: [
  errStr := server lastErrorString.
  server close.
  self halt: errStr.
  ].

server port == portNum ifFalse:[ self halt: 'bad port number' ] .
GsFile gciLog: 'Waiting for GsSecureSocket startTlsClientExample to connect...' 
	onClient: logToGciClient .
[server readWillNotBlockWithin: 10000] whileFalse: [
  GsFile gciLog: 'waiting...' onClient: logToGciClient
].

GsFile gciLog: 'Client connected, starting accept.' onClient: logToGciClient .
client := server accept .
GsFile gciLog: 'accept finished. client = ', client asString  onClient: logToGciClient .
client ifNil: [
  errStr := server lastErrorString.
  server close.
  GsFile gciLog: 'error from accept, ', errStr onClient: logToGciClient .
  self halt: errStr.
].
client linger: true length: 10.  "wait after closing until data is processed"
dataString := String new.
resultString := String withAll: 'OK'.
resultString add: (Character withValue: 0).
starttls := false.

"Keep receiving normal data and sending OK back until we see STARTTLS"
[starttls] whileFalse:[
  [ client readWillNotBlockWithin: 10000] whileFalse: [ 
    GsFile gciLog: 'Waiting for client to write...' onClient: logToGciClient .
  ].
  dataString size: 0.
  GsFile gciLog: 'Got something from the client to read.' onClient: logToGciClient .
  [
    chunk := client readString: 4000 "max bytes".
    chunk ifNil:[ self halt: 'Error in reading from socket' ].
    dataString addAll: chunk .
    (chunk at: chunk size) == (Character withValue: 0)  "until null terminator"
  ] untilTrue .
  
  sleepTime := 0 .
  [ client writeWillNotBlock ] whileFalse:[ 
    sleepTime == 0 ifTrue:[ 
       GsFile gciLog: 'waiting because write to client would block' onClient: logToGciClient.
    ].
    sleepTime > 5000 ifTrue:[
      GsFile gciLog: 'ERROR, client would block' onClient: logToGciClient .
      self halt: 'socket write will block' 
    ].
    System _sleepMs: 20 .
    sleepTime := sleepTime + 20 .
  ].
  "Got all the data, send back OK"
  numWritten := client write: resultString .
  GsFile gciLog: 'wrote ', numWritten asString, ' bytes'  onClient: logToGciClient .
  numWritten == resultString size ifFalse:[ self halt: 'error writing to socket'].
  starttls := (dataString findString: 'STARTTLS' startingAt: 1) ~~ 0 .
].

"Client has sent STARTTLS. Create new SSL server socket using existing client
 connection.  Socket file descriptor is transferred to sslServer."
sslServer := GsSecureSocket newServerFromGsSocket: client .
sslServer ifNil:[ self halt: 'Failed to create GsSecureSocket from GsSocket'].
client close. "will not close the underlyding file descriptor"

"do not request a certificate from the client"
sslServer disableCertificateVerification.

classFailBlock := [:aClass :shouldLog | | arr |
  arr :=  aClass fetchErrorStringArray .
  1 to: arr size do: [:n | GsFile gciLog: (arr at: n) onClient: shouldLog  ].
  SocketError signal: 'Primitive failed' .
].

failBlock := [:sock :shouldLog| | err arr |
  err := sock fetchLastIoErrorString .
  err ifNotNil: [GsFile gciLog: err onClient: shouldLog].
  arr := sock class fetchErrorStringArray .
  1 to: arr size do:[:n| GsFile gciLog: (arr at: n) onClient: shouldLog ].
  sock close.
  SocketError signal: err asString.
].

"Example of using an encrypted private key that requires a passphrase."
(sslServer useCertificateFile: '$GEMSTONE/examples/openssl/certs/server.pem'
  withPrivateKeyFile: '$GEMSTONE/examples/openssl/certs/serverkey.pem'
  privateKeyPassphrase: 'SERVERKEY' )
  ifFalse:[ 
    GsFile gciLog: 'useCertificateFile failed.' onClient: logToGciClient .
    classFailBlock value: GsSecureSocket value: logToGciClient.
].

"Use all ciphers except NULL ciphers and anonymous Diffie-Hellman
 and sort by strength."
(sslServer setCipherListFromString: 'ALL:!ADH:@STRENGTH')
  ifFalse:[ GsFile gciLog: 'setCipherListFromString: failed'  
                  onClient: logToGciClient
  ].

GsFile gciLog: 'Waiting for client to secureConnect: ...' onClient: logToGciClient .
sslServer secureAccept
  ifTrue:[ GsFile gciLog: 'Secure connection established.' 
                  onClient: logToGciClient ]
  ifFalse:[ | certError |
    GsFile gciLog: 'secureAccept failed.'  onClient: logToGciClient.
    certError := sslServer class fetchLastCertificateVerificationErrorForServer .
    certError ifNil: [  
        GsFile gciLog: 'nil result from fetchLastCertificateVerificationErrorForServer' 
           onClient: logToGciClient
    ] ifNotNil:[
        GsFile gciLog: ('Cert error: =>' , certError) 
           onClient: logToGciClient.].
    failBlock value: sslServer value: logToGciClient.
].

"If we get here, we have a secure connection to the client."

GsFile gciLog: ('Current cipher in use is: ', sslServer fetchCipherDescription)
       onClient: logToGciClient.

server close .
sslServer close .
^ true
%

category: 'Examples'
classmethod: GsSecureSocket
startTlsClientExample: logToGciClient usingPort: portNum address: serverAddress

"logToGciClient is a Boolean, true if GsFile-based logging is to be done to
 the RPC Gci client, false if to the server process's log file. Use false if
 executing via topaz 'nbrun'  .

 This client will connect to a server created with serverStartTlsExample.
 Then it will send 4 strings to the server and string 'OK' back to indicate
 the data was received.  Then the client will send the string 'STARTTLS' to
 tell the server to switch to encrypted communication. Once it receives back
 the 'OK' string, it will create a GsSecureSocket from the GsSocket and call
 secureConnect to establish a TLS connection. It will then print the cipher
 being used and close the connection.
   
 All strings sent and received are expected to be NULL-terminated.

 The server should already be listening for connections when this method is
 invoked. Start the server before starting the client."

| socket dataString  chunk  sleepCount outString sslClient failBlock |

socket := GsSocket new.
(socket connectTo: portNum on: serverAddress) ifFalse:[ 
  socket close .
  self halt: 'Unable to connect to port ' , portNum asString ,
    '. Wait 30 seconds or so, then ensure startTlsServerExample is started first.' 
  ].
dataString := String new .
outString := String new.

"Exchange some unencrypted data messages"
1 to: 5 do:[:n| | numWritten|
  sleepCount := 0.
  [ socket writeWillNotBlockWithin: 5000] whileFalse: [ 
    GsFile gciLog: 'Waiting to write to server...' onClient: logToGciClient .
    sleepCount := sleepCount + 1.
    sleepCount > 200 ifTrue:[
      GsFile gciLog: 'Not ready to write after 20 sec' onClient: logToGciClient.
      self halt: 'Not ready to read after 20 sec' .
    ].
    System _sleepMs: 100 . "sleep 100 ms then retry"
  ].

  outString size: 0.
  n == 5
    ifTrue:[ outString addAll: 'STARTTLS']
    ifFalse:[ outString addAll: DateTime now asString] .
  outString add: (Character withValue: 0).
  
  numWritten := socket write: outString.
  GsFile gciLog: 'wrote ', numWritten asString, ' bytes'  onClient: logToGciClient .
  numWritten == outString size ifFalse:[ self halt: 'error writing to socket'].
  [ socket readWillNotBlockWithin: 10000] whileFalse: [
      GsFile gciLog: 'Waiting for server to write...' onClient: logToGciClient .
  ].
  
  GsFile gciLog: 'Got something from the server to read.' onClient: logToGciClient .
  dataString size: 0.
  [
    chunk := socket readString: 4000.
    chunk ifNil:[ self halt: 'Error in reading from socket' ].
    dataString addAll: chunk .
    (chunk at: chunk size) == (Character withValue: 0)  "read until null terminator"
  ] untilTrue .
  n == 5 ifFalse:[ System _sleepMs: 250 ].
].

failBlock := [:sock :shouldLog| | err arr |
  err := sock fetchLastIoErrorString .
  err ifNotNil: [GsFile gciLog: err onClient: shouldLog].
  arr := sock class fetchErrorStringArray .
  1 to: arr size do:[:n| GsFile gciLog: (arr at: n) onClient: shouldLog ].
  sock close.
  SocketError signal: err asString.
].

"Setup CA certs to verify the server certs"
(GsSecureSocket useCACertificateFileForClients: 
      '$GEMSTONE/examples/openssl/certs/serverCA.pem')
  ifFalse:[GsFile gciLog: 'useCACertificateFileForClients failed'  
                  onClient: logToGciClient
  ].

"Transfer connection from socket to a new SSL socket."
sslClient:= GsSecureSocket newClientFromGsSocket: socket .
sslClient ifNil:[self halt: 'Error: newClientFromGsSocket: method failed'].
"close the old one. This will NOT close the file descriptor."
socket close.

"Request a cert from the server"
sslClient enableCertificateVerification .

"Set the cipher list for this SSL socket only.
 Use all ciphers except NULL ciphers and anonymous Diffie-Hellman
 and sort by strength."
(sslClient setCipherListFromString: 'ALL:!ADH:@STRENGTH')
  ifFalse:[ GsFile gciLog: 'setCipherListFromString: failed'  
                  onClient: logToGciClient
  ].

"Attempt to establish a secure connection to the SSL server..."
sslClient secureConnect
  ifTrue:[ GsFile gciLog: 'Secure connection established'
                  onClient: logToGciClient]
  ifFalse:[ |certError|
    GsFile gciLog: 'secureConnect failed' onClient: logToGciClient.
    certError := GsSecureSocket fetchLastCertificateVerificationErrorForClient .
    certError ifNil: [  
       GsFile gciLog: 'nil result from fetchLastCertificateVerificationErrorForClient' 
              onClient: logToGciClient
        ] ifNotNil: [
        GsFile gciLog: ('Cert error: =>' , certError) onClient: logToGciClient].
  failBlock value: sslClient value: logToGciClient.

  ].

"We have a secure connection to the server if we get here."
GsFile gciLog: ('Current cipher in use is: ', sslClient fetchCipherDescription)
       onClient: logToGciClient.

sslClient close.
^ true. 
%
