! file  sortedcollection2.gs

category: 'Private'
method: SortedCollection
resort

"Re-sorts the receiver according to its sortBlock."

| saveArray |
self size > 1 ifTrue: [
  saveArray := Array withAll: self.
  [ self _basicSize: 0 .
    self addAll: saveArray 
  ] onException: Error do: [:ex |
    "fix bug 39227/41089"
    System disableCommitsUntilAbortWithReason: 'Error during SortedCollection>>resort'.
    ex pass
  ].
].
%

category: 'Repository Conversion'
classmethod: SortedCollection
convertAll: anArray usingCache: dict hiddenSetForErrors: hsIdx

"Convert an Array of SortedCollections by recompiling their sort blocks from
 SimpleBlocks to ExecBlocks.  anArray must contain only SortedCollections.
 dict is used as a cache to canonicalize sort blocks with the same source 
 string.

 Caller must handle and catch AlmostOutOfMemory errors and handle commits
 and aborts."

| sbCls errorBlock oldIcuDefaultMth newIcuBlk |
sbCls := Globals at:#SimpleBlock ifAbsent:[ (Globals at:#ObsoleteClasses) at: #SimpleBlock ] .
newIcuBlk := IcuSortedCollection new _defaultBlock . "a 3 arg block"
oldIcuDefaultMth := Globals at:#IcuSortedCollection_oldDefaultMethod "meth for old 2 arg block"
                    ifAbsent:[ newIcuBlk := nil . nil ].
errorBlock := [ :_sc :_hsIdx :msg | | sys |
     GsFile gciLogServer:'SortedCollection oop ', _sc asOop asString , msg .
     (sys := System) _add: _sc to: _hsIdx ;
                     _sessionCacheStatAt: 1 put: (sys _hiddenSetSize: _hsIdx) 
    ].
1 to: anArray size do:[: ofs | | aSortedColl aBlock cls |
  aSortedColl := anArray at: ofs .
  aBlock := aSortedColl sortBlock .
  cls := aBlock class .
  (aSortedColl isKindOf: IcuSortedCollection) ifTrue:[
    aBlock method == oldIcuDefaultMth ifTrue:[
      aSortedColl _sortBlock: newIcuBlk .    
      cls := nil "don't need recompile "
    ] ifFalse:[ 
      aBlock argumentCount == 3 ifFalse:[
	errorBlock value: aSortedColl value: hsIdx 
                value:' an IcuSortedCollection, 2 arg block must be converted to 3 arg block'.
        cls := nil "disable recompile attempt"
      ].
    ].
  ].
  cls ifNotNil:[
    (cls _subclassOf: ExecBlock) ifTrue:[
      aBlock needsRecompile ifTrue:[
	[ aBlock recompile  .
	  "successful recompilation for block's method,
	   this block will show up as needsRecompile==false if seen again"
	] onSynchronous: Error do:[ :ex|
	  errorBlock value: aSortedColl value: hsIdx value: 
             ' block recompile failed,' , ex asString 
	].
      ].
    ] ifFalse:[ 
      cls == sbCls ifTrue:[ "SimpleBlock from 2.x " | str val |
	str := aBlock _sourceString .
	(val := dict at: str otherwise: nil) 
	  ifNotNil:[ aSortedColl _sortBlock: val ]
	  ifNil:[ "A miss on the dictionary, compile it now"
	    [ val := str evaluate .
	      val ifNotNil:[ dict at: str put: val. "Add to cache" 
			  aSortedColl _sortBlock: val ]
		  ifNil:[ errorBlock value: aSortedColl value: hsIdx 
			  value: ' evaluate of recompiled block failed' ].
	    ]  onSynchronous: Error do:[:ex | 
               errorBlock value: aSortedColl value: hsIdx 
                 value: ' block recompile failed, ' , ex asString
	    ] .
	  ].
      ] ifFalse:[
	errorBlock value: aSortedColl value: hsIdx value:' block is a v2.x ComplexBlock'.
      ].
    ].
  ].
].
%

